/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Authors:
 *     Akos Frohner <Akos.Frohner@cern.ch>
 */

#include <stdio.h>                  /* for NULL */
#include <stdlib.h>                 /* for free() */
#include <string.h>                 /* for strcmpy() and strlen() */
#include <stdarg.h>                 /* for va_list */
#include <time.h>                   /* for time/localtime/strftime */

#include "ServiceDiscoveryI.h"

#define PRIVATE static
#define PUBLIC /* empty */

static SD_I_log4c_LogLevel SD_I_log4c_current_loglevel = SD_I_LOG4C_UNKNOWN;
static FILE *SD_I_log4c_current_logfile = NULL;

static char *SD_I_log4c_loglevel_names[] = {
    "DEBUG",
    "INFO",
    "WARN",
    "ERROR"
};

#ifndef __GNUC__
#define __attribute__(x)
#endif
void SD_I_log4c_init(void) __attribute__((__constructor__));
void SD_I_log4c_done(void) __attribute__((__destructor__));

PUBLIC void SD_I_log4c_init(void) {
    char *envar;

    SD_I_log4c_current_loglevel = SD_I_LOG4C_NONE;

    envar = getenv(SDEnv_LOG_LEVEL);
    if (envar != NULL) {
        if(strncmp(envar, SD_I_log4c_loglevel_names[SD_I_LOG4C_DEBUG], 
                   strlen(SD_I_log4c_loglevel_names[SD_I_LOG4C_DEBUG])) == 0) {
            SD_I_log4c_current_loglevel = SD_I_LOG4C_DEBUG;
        }
        else if(strncmp(envar, SD_I_log4c_loglevel_names[SD_I_LOG4C_INFO], 
                        strlen(SD_I_log4c_loglevel_names[SD_I_LOG4C_INFO])) == 0) {
            SD_I_log4c_current_loglevel = SD_I_LOG4C_INFO;
        }
        else if(strncmp(envar, SD_I_log4c_loglevel_names[SD_I_LOG4C_WARN], 
                        strlen(SD_I_log4c_loglevel_names[SD_I_LOG4C_WARN])) == 0) {
            SD_I_log4c_current_loglevel = SD_I_LOG4C_WARN;
        }
        else if(strncmp(envar, SD_I_log4c_loglevel_names[SD_I_LOG4C_ERROR], 
                        strlen(SD_I_log4c_loglevel_names[SD_I_LOG4C_ERROR])) == 0) {
            SD_I_log4c_current_loglevel = SD_I_LOG4C_ERROR;
        }
        else {
            SD_I_log4c_current_loglevel = SD_I_LOG4C_NONE;
        }
    }

    if (SD_I_log4c_current_loglevel < SD_I_LOG4C_NONE) {
        envar = getenv(SDEnv_LOG_FILE);
        if (envar != NULL) {
            SD_I_log4c_current_logfile = fopen(envar, "a+");
            // returns NULL on error, in which case we will
            // log to the stderr anyway
        }
    }
}

PUBLIC void SD_I_log4c_done(void) {
    if (SD_I_log4c_current_logfile != NULL) {
        fclose(SD_I_log4c_current_logfile);
        SD_I_log4c_current_logfile = NULL;
    }
}

/* The goal: 2006-06-27 10:40:01,655 */
#define SD_I_LOG4C_TIME_FORMAT "%F %T"
#define SD_I_LOG4C_TIME_LENGTH 25
static char SD_I_log4c_time[SD_I_LOG4C_TIME_LENGTH];

PUBLIC void SD_I_log4c_printf(SD_I_log4c_LogLevel loglevel, 
    const char *file, const char *function, const int line,
    const char *format, ...) 
{
    if (loglevel < 0 || loglevel > SD_I_LOG4C_ERROR) return;
    if (SD_I_log4c_current_loglevel > loglevel) return;

    va_list ap;
    FILE *logfile = SD_I_log4c_current_logfile;
    time_t logt;
    struct tm *logtm;
    const char *basename;

    if (NULL == logfile) logfile = stderr;
    
    logt = time(NULL);
    logtm = localtime(&logt);
    if (logtm == NULL) {
        SD_I_log4c_time[0] = '\0';
    }
    else if(strftime(SD_I_log4c_time, sizeof(SD_I_log4c_time), SD_I_LOG4C_TIME_FORMAT, logtm) == 0) {
        SD_I_log4c_time[0] = '\0';
    }
    
    basename = rindex(file, '/');
    if (NULL != basename) {
        basename++; // skip to after the slash
    }
    else {
        basename = file;
    }

	va_start(ap, format);
    fprintf(logfile, "%s %s - ", SD_I_log4c_time, SD_I_log4c_loglevel_names[loglevel]);
    vfprintf(logfile, format, ap);
    fprintf(logfile, " - %s#%s:%d\n", basename, function, line);
    fflush(logfile);
	va_end(ap);
}

/* End of file. */
